/** 
 * Written by:
 *		Jun Peng (junpeng@stanford.edu)
 *		Kincho Law (law@stanford.edu)
 * ----------------------------------
 * Define the basic funcation to retrieve data from MySQL database.
 */ 


import java.sql.*;
import java.util.*;
import java.io.*;
import java.text.DateFormat;


public class DBAccess {

	public static Connection dbConn = null;

	static final String dbUsr = MyConsts.dbUsr;
	static final String dbPwd = MyConsts.dbPwd;
	static final String dbConnStr = MyConsts.dbConnStr;
    

	/**
	 * Connect to the database, update the value of dbConn.
	 */
	public void initdb(PrintWriter out) 
	{
		try 
		{
			Class.forName("com.mysql.jdbc.Driver").newInstance();
			dbConn = DriverManager.getConnection(dbConnStr, dbUsr, dbPwd);
		}
		catch (Exception e)
		{
			out.println("Database connection failed!");
		}
	}

	/**
	 * Close the connection to database.
	 */
	public void closedb(PrintWriter out)
	{
		try 
		{
			dbConn.close();
		}
		catch (Exception e)
		{
			out.println("Cannot close database connection!");
		}
	}

	/**
	 * Based on the input fileID, retrive the fileURI value.
	 */
	public String getFileURI(long fileID)
	{
		String fileURI = new String();

		try 
		{
			Statement stmt = dbConn.createStatement();
			
			String queryString = "SELECT uri FROM File WHERE fileID = " + fileID;
			ResultSet rSet = stmt.executeQuery(queryString);
			
			while (rSet.next()) 
			{
				fileURI = rSet.getString(1);
			}
		} 
		catch (SQLException e) 
		{
			e.printStackTrace();
		}	

		return fileURI;
	}


	public Vector getColumnValues(String theTable, String theColumn)
	{
		Vector cValues = new Vector();
		
		try 
		{
			Statement stmt = dbConn.createStatement();
			
			String queryString =  "SELECT " + theColumn + " FROM " + theTable;
			ResultSet rSet = stmt.executeQuery(queryString);
			
			while (rSet.next()) 
			{
				String oneValue = rSet.getString(1);
				cValues.addElement(oneValue);
			}
		} 
		catch (SQLException e) 
		{
			e.printStackTrace();
		}

		return cValues;
	}

	/**
	 * Get the primary key values (column keyColumn) based on the foreign key
	 * value to satisfy (theColumn = theValue). 
	 */
	public Vector getCurValues(String theTable, String theColumn, long theValue, String keyColumn)
	{
		Vector curKeys = new Vector();

		try 
		{
			Statement stmt = dbConn.createStatement();
			
			String queryString =  "SELECT " + keyColumn + " FROM " + theTable + 
				" WHERE " + theColumn + " = \'" + theValue + "\'";

			ResultSet rSet = stmt.executeQuery(queryString);
			
			while (rSet.next()) 
			{
				String oneKey = rSet.getString(1);
				curKeys.addElement(oneKey);
			}
		} 
		catch (SQLException e) 
		{
			e.printStackTrace();
		}

		return curKeys;
	}


	/**
	 * Get the primary key values (column keyColumn) based on the foreign key
	 * value to satisfy (theColumn = theValue). 
	 */
	public Vector getCurKeys(String theTable, String theColumn, long theValue, String keyColumn)
	{
		Vector curKeys = new Vector();

		try 
		{
			Statement stmt = dbConn.createStatement();
			
			String queryString =  "SELECT " + keyColumn + " FROM " + theTable + 
				" WHERE " + theColumn + " = \'" + theValue + "\'";

			ResultSet rSet = stmt.executeQuery(queryString);
			
			while (rSet.next()) 
			{
				long oneKey = rSet.getLong(1);
				curKeys.addElement(new Long(oneKey));
			}
		} 
		catch (SQLException e) 
		{
			e.printStackTrace();
		}

		return curKeys;
	}

	/**
	 * Retrieve the primary key value of theTable based on theColumn = theValue.
	 */
	public long getPrimaryKey(String theTable, String theColumn, String theValue, String pKey)
	{
		long primaryKey = 0;

		try 
		{
			Statement stmt = dbConn.createStatement();
			
			String queryString =  "SELECT " + pKey + " FROM " + theTable + 
				" WHERE " + theColumn + " = \'" + theValue + "\'";
			ResultSet rSet = stmt.executeQuery(queryString);
			
			while (rSet.next()) 
			{
				primaryKey = rSet.getLong(1);
			}
		} 
		catch (SQLException e) 
		{
			e.printStackTrace();
		}
		return primaryKey;
	}


	/**
	 * Retrieve the tuples from theTable where theColumn = theValue.
	 */
	public void getTuplesByString(String theTable, String theColumn, String theValue, PrintWriter out)
	{
		int tableSize = 0;
		Vector theField = new Vector();
		Vector theType = new Vector();

		try 
		{
			Statement stmt = dbConn.createStatement();
			
			String queryString = "DESCRIBE " + theTable ;
			ResultSet schemaSet = stmt.executeQuery(queryString);
			
			while (schemaSet.next()) 
			{
				theField.addElement( schemaSet.getString(1) );
				theType.addElement( schemaSet.getString(2) );
				tableSize++;
			}
		} 
		catch (SQLException e) 
		{
			e.printStackTrace(out);
		}

		try 
		{
			Statement stmt = dbConn.createStatement();
			
			String queryString = "SELECT * FROM " + theTable + 
				" WHERE " + theColumn + " = \'" + theValue + "\'";
			ResultSet rSet = stmt.executeQuery(queryString); 

			out.println("<table border=1>");
			out.println("<tr> <td><b>SLOT</b></td>\n" +
				"     <td><b>VALUE</b></td> </tr>");

			while (rSet.next())
			{
				for (int i = 0; i < tableSize; i++) 
				{
					out.println("<tr> <td>" + theField.elementAt(i) + "</td>");
					String result1 = rSet.getString(i+1);
					out.println("<td> " + result1 + "</td></tr>");
				}
			}
			out.println("</table>");
		} 
		catch (SQLException e) 
		{
			e.printStackTrace(out);
		}
		
	}

	/**
	 * Retrieve the localTimeZone, startDate and endDate information.
	 */
	public void getDate(String theTable, String theColumn, long theValue, PrintWriter out, int isDate)
	{
		try 
		{
			Statement stmt = dbConn.createStatement();
			
			String queryString = null;
			if (isDate == 1) 
			{
				queryString =  "SELECT title, localTimeZone, startDate, endDate FROM " 
					+ theTable + " WHERE " + theColumn + " = \'" + theValue + "\'";
			} 
			else 
			{
				queryString =  "SELECT title, localTimeZone, startDateTime, endDateTime FROM " 
					+ theTable + " WHERE " + theColumn + " = \'" + theValue + "\'";
			}

			ResultSet rSet = stmt.executeQuery(queryString);
			
			while (rSet.next()) 
			{
				String title = rSet.getString(1);
				String timeZone = rSet.getString(2);
				java.util.Date startDate = rSet.getDate(3);
				java.util.Date endDate = rSet.getDate(4);
				out.println("<p>The Activity <b>" + title + "</b>");
				out.println("happened at time zone <b>" + timeZone + "</b>:<br>");

				if (isDate == 1) 
				{
					out.println("startDate: " + DateFormat.getDateInstance().format(startDate));
					out.println(" | endDate: " + DateFormat.getDateInstance().format(endDate));
				} 
				else 
				{
					out.println("startDateTime: " + DateFormat.getDateTimeInstance().format(startDate));
					out.println(" | endDateTime: " + DateFormat.getDateTimeInstance().format(endDate));
				}
			}
		} 
		catch (SQLException e) 
		{
			e.printStackTrace();
		}
	}


	/**
	 * Retrieve the tuples from theTable where theColumn = theValue.
	 */
	public void getTuplesByKey(String theTable, String theColumn, long theValue, PrintWriter out)
	{
		int tableSize = 0;
		Vector theField = new Vector();
		Vector theType = new Vector();

		try 
		{
			Statement stmt = dbConn.createStatement();
			
			String queryString = "DESCRIBE " + theTable ;
			ResultSet schemaSet = stmt.executeQuery(queryString);
			
			while (schemaSet.next()) 
			{
				theField.addElement( schemaSet.getString(1) );
				theType.addElement( schemaSet.getString(2) );
				tableSize++;
			}
		} 
		catch (SQLException e) 
		{
			e.printStackTrace(out);
		}

		try 
		{
			Statement stmt = dbConn.createStatement();
			
			String queryString = "SELECT * FROM " + theTable + 
				" WHERE " + theColumn + "=" + theValue;
			ResultSet rSet = stmt.executeQuery(queryString);

			out.println("<table border=1>");
			out.println("<tr> <td><b>SLOT</b></td>\n" +
				"     <td><b>VALUE</b></td> </tr>");
			
			while (rSet.next()) 
			{
				for (int i = 0; i < tableSize; i++) 
				{
					out.println("<tr> <td>" + theField.elementAt(i) + "</td>");
					String result1 = rSet.getString(i+1);
					out.println("<td> " + result1 + "</td></tr>");
				}
			}
			out.println("</table>");
		} 
		catch (SQLException e) 
		{
			e.printStackTrace(out);
		}
	}


	/**
	 * A function used for testing only.
	 */
	public static void main(String []args) 
	{
		DBAccess da = new DBAccess();
		da.initdb(new PrintWriter(System.out));
		//da.getTuplesByKey("Task", "projectID", 100001, System.out);
		//System.out.println(da.getFileURI(10000001));
		da.getTuplesByString("Project", "title", "miniMOST-1", new PrintWriter(System.out));

		Vector tasks = da.getCurKeys("Task", "projectID", 100001, "taskID");

		System.out.println(((Long)(tasks.elementAt(0))).longValue());
	}

}