/** 
 * Written by:
 *		Jun Peng (junpeng@stanford.edu)
 *		Kincho Law (law@stanford.edu)
 * ----------------------------------
 * ReportGenDB defines the functions for generating engineering report.
 * Data are saved in MySQL database.
 */ 


import java.sql.*;
import java.io.*;
import java.util.*;

import javax.servlet.*;
import javax.servlet.http.*;


public class DBDataRetrieval extends HttpServlet 
{

	DBMapping theMap = new DBMapping();


	/* Depends on the user's choice, we will take different action to 
	 * display a portion of the project-related data.
	 */
	public void handleGenReport(String project, String category, PrintWriter out)
	{       
		if (category.equals("summary")) 
		{
			theMap.getSummary(project, out); 
		}
		else if (category.equals("scope")) 
		{
			theMap.getScope(project, out);
		}
		else if (category.equals("setupoverview")) 
		{
			theMap.getSetupOverview(project, out);
		} 
		else if (category.equals("infrastructure")) 
		{
			theMap.getInfrastructure(project, out);
		}
		else if (category.equals("specimen")) 
		{
			theMap.getSpecimen(project, out);
		} 
		else if (category.equals("loading")) 
		{
			theMap.getLoading(project, out);
		} 
		else if (category.equals("instrumentation")) 
		{
			theMap.getInstrumentation(project, out);
		}
		else if (category.equals("schedule")) 
		{
			theMap.getSchedule(project, out);
		} 
		else if (category.equals("results")) 
		{
			theMap.getResults(project, out);
		}
		else if (category.equals("dataprocessing")) 
		{
			theMap.getDataProcessing(project, out);
		} 
		else if (category.equals("predications")) 
		{
			theMap.getPredications(project, out);
		}

	}


	public void genReport(PrintWriter out)
	{
		out.println("<p>\n" +
			"<h3>Engineering Report Generation Facility:</h3>");
		out.println("<form method=\"GET\">");
        
		out.println("<b>Please choose the Project first:</b><br>\n" +
			"<select name=\"THEPROJECT\">\n" +
			"\t<option value=>\n");

		DBAccess dbAccess = theMap.getDBAccess();
		Vector allProjects = dbAccess.getColumnValues("Project", "title");
		for (int i = 0; i < allProjects.size(); i++) 
		{
			String projName = (String)allProjects.elementAt(i);
			out.println("\t<option value=\"" + projName + "\">" + projName);
		}

		out.println("</select> <br>\n");
        
		out.println("<b>Choose the category that you want to download:</b><br>\n" +
			"<select name=\"THECHOICE\">\n" +
			"\t<option value=\"summary\">1. Summary\n" +
			"\t<option value=\"scope\">2. Scope and general presentation\n" + 
			"\t<option value=\"setupoverview\">3. Test setup overview\n" + 
			"\t<option value=\"infrastructure\">3.1. Infrastructure setup\n" + 
			"\t<option value=\"specimen\">\t3.2. Specimen description\n" +
			"\t<option value=\"loading\">3.3. Loading System\n" + 
			"\t<option value=\"instrumentation\">3.4. Instrumentation\n" +
			"\t<option value=\"schedule\">4. Test schedule\n" +
			"\t<option value=\"results\">5. Test results\n" +
			"\t<option value=\"dataprocessing\">6. Data processing\n" + 
			"\t<option value=\"predictions\">7. Analytical predictions\n" +
			"</select> <p>\n");
        
		out.println("<input type=\"submit\" value=\"Submit\" name=\"ReportChoice\">\n" +
			"</form><br>" ); 	
	}
    

	public void printHeading(PrintWriter out, String heading)
	{
		out.println("<html>");
		out.println("<body bgcolor=\"white\">");
		out.println("<head><H2>" + heading +  "</H2></head>");
		out.println("<hr>");
	}


	public void doGet(HttpServletRequest request,
		HttpServletResponse response)
		throws IOException, ServletException
	{
		response.setContentType("text/html");
		PrintWriter out = response.getWriter();
		DBAccess dbAccess = theMap.getDBAccess();
		dbAccess.initdb(out);
	
		if (request.getParameter("ReportChoice") != null)
		{
			printHeading(out, "Engineering Report Generation Facility:");
			handleGenReport(request.getParameter("THEPROJECT"), 
					request.getParameter("THECHOICE"), out);
		}
		else 
		{
			printHeading(out, "The Retrieving of NEESMeta Objects:");
			genReport(out);
		}
   
		dbAccess.closedb(out);
		out.println("</body></html>");
	}

	public void doPost(HttpServletRequest request,
		HttpServletResponse response)
		throws IOException, ServletException
	{
		doGet(request, response);
	}

}